package pstsp.support.basic;

import org.openqa.selenium.WebDriver;
import pstsp.support.selenium_utils.Click;
import pstsp.support.test_oracle.OracleInicialization;
import pstsp.support.tools.Drivers;
import pstsp.support.tools.ProjectLogger;

import javax.swing.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Properties;

/**
 * Configurations class
 * <p>
 * Serves for an initial settings of all configuration
 * the basic file is the {@link Configurations#CONFIGURATIONS_FILE}
 *
 * @author Pavel Herout
 * @version 2022-02-09
 */

public class Configurations {

  private static final String CONFIG_PROPERTY_NAME = "pstsp.configuration.uri";

  /**
   * Build number
   * it is read from Home applications page
   */
  public static String BUILD_NUMBER = "N/A";

  /**
   * the basic configuration file - it must exist and must be correct
   */
  private static final String CONFIGURATIONS_FILE = "configurations.txt";

  /**
   * default directory for all configuration files
   * directory can be changed in environment
   * by {@code -Dpstsp.configuration.uri=path/to/this/configuration/file/}
   */
  private static final String DEFAULT_DIRECTORY = ".";

  /**
   * the reference to the Singleton design pattern
   */
  private static Configurations INSTANCE = new Configurations();

  /**
   * used directory for all configuration files
   * <p>
   * directory can be changed in {@link Configurations#CONFIGURATIONS_FILE}
   * by {@code -Dpstsp.configuration.uri=path/to/this/configuration/file/}
   */
  private String CONFIG_DIRECTORY;

  /**
   * Application properties overridden by System properties
   */
  private static Properties runtimeConfiguration;

  /**
   * Private constructor - design pattern Singleton
   * <ol>
   * <li>sets config directory</li>
   * <li>read {@link Configurations#CONFIGURATIONS_FILE} and sets all items into responsible classes</li>
   * </ol>
   */
  private Configurations() {
    //load configuration file setings
    String envDirectory = System.getenv(CONFIG_PROPERTY_NAME);
    String javaEnvDirectory = System.getProperty(CONFIG_PROPERTY_NAME);

    //java env variable overrides system env variable
    CONFIG_DIRECTORY = javaEnvDirectory != null ? javaEnvDirectory : envDirectory;

    //no definition = set default value
    if (CONFIG_DIRECTORY == null || CONFIG_DIRECTORY.isEmpty()) {
      CONFIG_DIRECTORY = DEFAULT_DIRECTORY;
    }
    //make config dir absolute
    CONFIG_DIRECTORY = new File(CONFIG_DIRECTORY).getAbsolutePath() + "/";
    if (CONFIG_DIRECTORY.endsWith("\\./") == true) {
      int end = CONFIG_DIRECTORY.length() - 2;
      CONFIG_DIRECTORY = CONFIG_DIRECTORY.substring(0, end);
    }

    String configFileURI = CONFIG_DIRECTORY + CONFIGURATIONS_FILE;

    Properties configProperties = new Properties();
    try {
      configProperties.load(new FileInputStream(configFileURI));
    } catch (IOException e) {
      e.printStackTrace();
      configurationHasNotBeenSuccessfull(envDirectory, javaEnvDirectory,
                                         e.getMessage(), true);
    }

    configProperties.putAll(System.getenv());
    configProperties.putAll(System.getProperties());

    runtimeConfiguration = configProperties;

    // settings from config file
    try {
      Url.BASE_URL = configProperties.getProperty("URL_Base");
      checkProperty(Url.BASE_URL, "URL_Base");
      Drivers.WEB_BROWSER_TYPE = configProperties.getProperty("WebBrowserType");
      checkProperty(Drivers.WEB_BROWSER_TYPE, "WebBrowserType");
      Drivers.MAXIMIZED_SCREEN = configProperties.getProperty("WebBrowserMaximizedScreen");
      checkProperty(Drivers.MAXIMIZED_SCREEN, "WebBrowserMaximizedScreen");
      Drivers.CHROME_DRIVER_URI = configProperties.getProperty("ChromeDriverURI");
      checkProperty(Drivers.CHROME_DRIVER_URI, "ChromeDriverURI");
      Drivers.OPERA_DRIVER_URI = configProperties.getProperty("OperaDriverURI");
      checkProperty(Drivers.OPERA_DRIVER_URI, "OperaDriverURI");
      Drivers.FIREFOX_DRIVER_URI = configProperties.getProperty("FirefoxDriverURI");
      checkProperty(Drivers.FIREFOX_DRIVER_URI, "FirefoxDriverURI");
      Drivers.EDGE_DRIVER_URI = configProperties.getProperty("EdgeDriverURI");
      checkProperty(Drivers.EDGE_DRIVER_URI, "EdgeDriverURI");
      Drivers.OPERA_EXE_URI = configProperties.getProperty("OperaExeURI");
      checkProperty(Drivers.OPERA_EXE_URI, "OperaExeURI");
      Drivers.CLOSE_BROWSER = configProperties.getProperty("CloseBrowser");
      checkProperty(Drivers.CLOSE_BROWSER, "CloseBrowser");
      ProjectLogger.LOG_FILE_CONFIG = configProperties.getProperty("LogFileConfig");
      checkProperty(ProjectLogger.LOG_FILE_CONFIG, "LogFileConfig");

      String timeoutStr = configProperties.getProperty("timeout");
      checkProperty(timeoutStr, "timeout");
      Click.TIMEOUT = Integer.parseInt(timeoutStr);
    } catch (Exception e) {
      e.printStackTrace();
      configurationHasNotBeenSuccessfull(envDirectory, javaEnvDirectory,
                                         e.getMessage(), true);
    }
  }

  /**
   * Static factory method - design pattern Singleton
   *
   * @return reference to Singleton
   */
  public static Configurations getInstance() {
    return INSTANCE;
  }

  /**
   * Returns loaded application properties or null
   *
   * @return current properties or null if not set
   */
  public static Properties getRuntimeConfiguration() {
    return runtimeConfiguration;
  }

  /**
   * Method reloads configuration based on actual property settings.
   *
   */
  public static void reconfigure(){
    INSTANCE = new Configurations();
  }

  /**
   * Sets all supporting parts (driver, logger, screenshots, etc.]
   * This method is called only from TestSetting on the very beginning of tests
   *
   * @return web driver
   */
  public WebDriver setConfiguration() {
    try {
      ProjectLogger.setLogger(CONFIG_DIRECTORY);
      ProjectLogger.logger.debug("Date_Time:        " + ProjectLogger.TEST_DATE_TIME);
      ProjectLogger.logger.debug("Config directory: " + CONFIG_DIRECTORY);
      Url.setConstantURLs();
      ProjectLogger.logger.debug("Base URL:         " + Url.BASE_URL);
      Drivers.setDriver();
      ProjectLogger.logger.debug("WebDriver:        " + Drivers.getDriver());
      Click.setWait(Drivers.getDriver());
      OracleInicialization.setAllFilesUri(CONFIG_DIRECTORY);
      OracleInicialization.restoreOracleStorage();
      readVersionsFromHomePage();
      ProjectLogger.logger.debug("Configuration has been successfully finished");
    } catch (Exception e) {
      e.printStackTrace();
      configurationHasNotBeenSuccessfull("", "",
              e.getMessage(), false);
    }
    return Drivers.getDriver();
  }

  /**
   * Reads versions from Home page
   */
  private static void readVersionsFromHomePage() {
//    BUILD_NUMBER = Home_Actions.readBuildNumber();
    // ToDo
    BUILD_NUMBER = "0.2.0";
  }


  /**
   * Error message window
   */
  private void configurationHasNotBeenSuccessfull(String envDirectory,
                                   String javaEnvDirectory, String eMessage, boolean readingConfig) {
    final String UNICODE_BULLET = "  \u2022 ";

    String envDir = (envDirectory == null || envDirectory.isEmpty()) ? "not set" : envDirectory;
    String javaDir = (javaEnvDirectory == null || javaEnvDirectory.isEmpty()) ? "not set" : javaEnvDirectory;
    String eMes = (eMessage == null || eMessage.isEmpty()) ? "not available" : eMessage;

    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd_HH:mm:ss");
    String date = LocalDateTime.now().format(formatter);

    String message = "Settings or default settings:\n"
            + UNICODE_BULLET + "EnvDirectory: " + envDir + "\n"
            + UNICODE_BULLET + "JavaEnvDirectory: " + javaDir + "\n"
            + UNICODE_BULLET + "ConfigDirectory: " + CONFIG_DIRECTORY + "\n\n"
            + "Exception error message:\n"
            + eMes + "\n\n";
    if (readingConfig == false) {
      message = message
              + "At least one of these configuration files in directory " + CONFIG_DIRECTORY + "\n"
              + UNICODE_BULLET + CONFIGURATIONS_FILE + "\n"
              + UNICODE_BULLET + OracleInicialization.USERS_FILE_NAME + "\n"
              + "is missing or has been corrupted or has an invalid content.\n\n"
              + "See ACTUAL (eg. " + date + ") log files 'test-config-log.txt'\n"
              + "and 'test-result-log.txt' for details. ";
    }

    JOptionPane.showMessageDialog(
            null,
            message,
            "Problems in reading the configuration files",
            JOptionPane.ERROR_MESSAGE);

    System.exit(1);
  }

  /**
   *
   * @param target constant in PSTSP system
   * @param configSource property name in configuration.txt
   */
  private static void checkProperty(String target, String configSource) {
    if (target == null) {
      throw new IllegalArgumentException(configSource
              + " in '" + CONFIGURATIONS_FILE + "' is not set");
    }
  }
}
