package pstsp.support.pageobjects.login;

/**
 * Login page
 *
 * @author David Kuta
 * @version 2022-01-20
 */

import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.FindBy;
import pstsp.support.basic.Const;
import pstsp.support.basic.Url;
import pstsp.support.pageobjects.PageObject;
import pstsp.support.pageobjects.commonparts.HeaderBar_Page;
import pstsp.support.selenium_utils.Click;
import pstsp.support.selenium_utils.MyAlert;
import pstsp.support.selenium_utils.Utils;
import pstsp.support.test_utils.Check;

import java.util.List;

public class Login_Page extends PageObject {
  // parts of the web page
//  public Header_Page header = new Header_Page();


  // ID's on web page
  public static final String USERNAME_INPUT = "frm-signInForm-username";
  public static final String PASSWORD_INPUT = "frm-signInForm-password";
  public static final String NM_LOGIN_BUTTON = "send";

  ////////////////////////////////////////////////////////////////
  ///  Warning: all texts serve for external testing purposses ///
  ////////////////////////////////////////////////////////////////

  // Texts on web page
  public static final String TXT_LOGIN_BUTTON = "Přihlásit";
  // placeholders' texts on web page
  public static final String TXT_USERNAME_INPUT_PLACEHOLDER = "Login";
  public static final String TXT_PASSWORD_INPUT_PLACEHOLDER = "Password";

///////////
  @FindBy(id = USERNAME_INPUT)
  private List<WebElement> usernameInput;

  /**
   * Get username input placeholder text
   *
   * @return username input placeholder text
   */
  public String getUsernameInputPlaceholderText() {
    String text = getElement(usernameInput, USERNAME_INPUT).getAttribute("placeholder").trim();
    return text;
  }

  /**
   * Get username
   *
   * Warning: usually will be empty
   *
   * @return username
   */
  public String getUsername() {
    String text = getElement(usernameInput, USERNAME_INPUT).getAttribute("value").trim();
    return text;
  }

  /**
   * Check username enabled
   *
   * @return true=enabled, false=disabled
   */
  public boolean isUsernameEnabled() {
    return getElement(usernameInput, USERNAME_INPUT).isEnabled();
  }

  /**
   * Set username
   *
   * @param username new username
   * @return LoginPage for chain commands
   */
  public Login_Page setUsername(String username) {
    WebElement element = getElement(usernameInput, USERNAME_INPUT);
    element.clear();
    element.sendKeys(username);
    return this;
  }


  ///////////
  @FindBy(id = PASSWORD_INPUT)
  private List<WebElement> passwordInput;

  /**
   * Get password input placeholder text
   *
   * @return password input placeholder text
   */
  public String getPasswordInputPlaceholderText() {
    String text = getElement(passwordInput, PASSWORD_INPUT).getAttribute("placeholder").trim();
    return text;
  }

  /**
   * Get password
   *
   * Warning: usually will be empty
   *
   * @return password
   */
  public String getPassword() {
    String text = getElement(passwordInput, PASSWORD_INPUT).getAttribute("value").trim();
    return text;
  }

  /**
   * Check password enabled
   *
   * @return true=enabled, false=disabled
   */
  public boolean isPasswordEnabled() {
    return getElement(passwordInput, PASSWORD_INPUT).isEnabled();
  }

  /**
   * Set password
   *
   * @param password new password
   * @return LoginPage for chain commands
   */
  public Login_Page setPassword(String password) {
    WebElement element = getElement(passwordInput, PASSWORD_INPUT);
    element.clear();
    element.sendKeys(password);
    return this;
  }

///////
  @FindBy(name = NM_LOGIN_BUTTON)
  private List<WebElement> loginButton;

  /**
   * Get login button text
   *
   * @return login button text
   */
  public String getLoginButtonText() {
    String text = getElement(loginButton, NM_LOGIN_BUTTON).getText().trim();
    return text;
  }

  /**
   * Check login button enabled
   *
   * @return true=enabled, false=disabled
   */
  public boolean isLoginButtonEnabled() {
    return getElement(loginButton, NM_LOGIN_BUTTON).isEnabled();
  }

  /**
   * Performs login activity and without any alert
   *  waits for header user name visible
   *  goes to XYZ page
   *
   * @return PageObject which must be changed to a propper type (GarantPage or MentorPage)
   */
  public PageObject clickOnLoginButton() {
    Click.clickAndWaitId(getElement(loginButton, NM_LOGIN_BUTTON),
            HeaderBar_Page.USER_NAME);
    return new PageObject();
  }

  /**
   * Tries login error activity
   *
   * @return
   */
  public void clickOnLoginButton_ErrorLogin() {
    Click.clickAndWaitURL(getElement(loginButton, NM_LOGIN_BUTTON),
            Url.URL_LOGIN);
    //Click.clickAndWaitId(getElement(loginButton, NM_LOGIN_BUTTON),USERNAME_INPUT);
    MyAlert alert = MyAlert.getAlert();
    String headerText = alert.getHeaderText();
    String bodyText = alert.getBodyText();
    Check.checkText("Chyba",headerText,MyAlert.MY_ALERT_ID);
    Check.checkText("Přihlášení se nezdařilo. Zkontrolujte kombinaci přihlašovacího jmena a hesla",bodyText, MyAlert.MY_ALERT_ID);
  }

  /**
   * Tries login error activity with blank username or password, checks for bootstrap form validation
   *
   */
  public void clickOnLoginButton_BlankInput(List<String> expectedMessage) {
    Click.clickAndWaitURL(getElement(loginButton, NM_LOGIN_BUTTON),
            Url.URL_LOGIN);
    Check.checkContentsOfTwoLists(Utils.getFormErrorText(),expectedMessage,"Error message " + expectedMessage + "was not found");
  }

}
