package pstsp.support.selenium_utils;

import org.openqa.selenium.*;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.FluentWait;
import org.openqa.selenium.support.ui.Wait;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.openqa.selenium.interactions.Actions;

import pstsp.support.basic.Const;
import pstsp.support.basic.Url;
import pstsp.support.pageobjects.PageObject;
import pstsp.support.pageobjects.commonparts.HeaderBar_Page;
import pstsp.support.pageobjects.commonparts.ModalClose_Page;
import pstsp.support.test_utils.Fail;
import pstsp.support.tools.Drivers;

import java.time.Duration;
import java.util.Collections;
import java.util.NoSuchElementException;

/**
 * Any clickable and waiting activity
 *
 * @author Pavel Herout
 * @version 2022-01-20
 */
public class Click {
  /** timeout 10 sec for maximum of waiting
   *  Warning: a less value is problematic for testing of a modal content
   */
  public static int TIMEOUT = Const.DEFAULT_TIMEOUT;

  /** wait object
   *  can be set as a 'normal' wait (setWait()) or 'fluent' wait (setFluentWait())
   *  in Configurations.setConfiguration()
   */
  public static Wait<WebDriver> wait;

  /**
   * Setting of a 'normal' wait
   *
   * @param driver web driver
   */
  public static void setWait(WebDriver driver) {
    wait = new WebDriverWait(driver, Duration.ofSeconds(TIMEOUT));
  }

  /**
   * Setting of a 'fluent' wait
   *
   * NOT actually used
   *
   * @param driver web driver
   */
  public static void setFluentWait(WebDriver driver) {
    wait = new FluentWait<WebDriver>(driver)
            .withTimeout(Duration.ofSeconds(TIMEOUT))
            .pollingEvery(Duration.ofMillis(500))
            .ignoring(NoSuchElementException.class);
  }


  /**
   * The "basic" activity - the first opening a web page at the URL
   * waits for Click.TIMEOUT
   * 
   * @param url URL of opened web page
   */
  public static void openUrlAndWait(String url) {
    Drivers.getDriver().get(url);
    wait.until(ExpectedConditions.urlToBe(url));
  }

  /**
   * Opening a web page by clicking at web element
   * 
   * @param clickable web element
   * @param expectedURL URL of opened web page
   */
  public static void clickAndWaitURL(WebElement clickable, String expectedURL) {
    clickable.click();
    wait.until(ExpectedConditions.urlToBe(expectedURL));
  }

  /**
   * Opening a first or a second web page by clicking at web element
   *
   * @param clickable web element
   * @param expectedURL_first URL of first possibly opened web page
   * @param expectedURL_second URL of second possibly opened web page
   */
  public static void clickAndWaitURL(WebElement clickable, String expectedURL_first,
                                     String expectedURL_second) {
    clickable.click();
    wait.until(
            ExpectedConditions.or(
                    ExpectedConditions.urlToBe(expectedURL_first),
                    ExpectedConditions.urlToBe(expectedURL_second)
            )
    );
  }

  /**
   * Opening a web page by clicking at web element
   *
   * @param clickable web element
   * @param expectedTitle title of opened web page
   */
  public static void clickAndWaitTitle(WebElement clickable, String expectedTitle) {
    clickable.click();
    wait.until(ExpectedConditions.titleIs(expectedTitle));
  }

  /**
   * Click at the button and wait until new expected web element with ID appears
   * 
   * @param button clickable button
   * @param idExpected ID of expected web element
   */
  public static void clickAndWaitId(WebElement button, String idExpected) {
//    clicableElementVisible(button);
    Click.clickAndWaitBy(button,By.id(idExpected));
  }

  /**
   * Click at the button and wait until new expected web element with By appears
   *
   * @param button clickable button
   * @param by location of expected web element
   */
  public static void clickAndWaitBy(WebElement button, By by) {
    button.click();
    try {
      wait.until(
              ExpectedConditions.visibilityOfAllElementsLocatedBy(by));
    }
    catch (TimeoutException e){
      if (new HeaderBar_Page().isApplicationErrorPresent()){
        Click.openUrlAndWait(Url.URL_ASSIGNMENT_LIST);
        Fail.failTestDueToWrongValue(HeaderBar_Page.INTERNAL_ERROR_ID,"Application is in error state, Internal error occurred.");
        return;
      }
    }
  }

  /**
   * Waiting for alert announcement
   *   usually used after JavaScript modal window confirmation
   * 
   * @param idSuccess ID of success alert
   * @param idError ID of error alert
   */
  public static void waitAlert(String idSuccess, String idError) {
    wait.until(
      ExpectedConditions.or(
          ExpectedConditions.visibilityOfAllElementsLocatedBy(By.id(idSuccess)),
          ExpectedConditions.visibilityOfAllElementsLocatedBy(By.id(idError))));
  }

  /**
   * Click at the button and wait for alert announcement
   * 
   * @param button clickable button
   * @param idSuccess ID of success alert
   * @param idError ID of error alert
   */
  public static void clickAndWaitAlert(WebElement button, String idSuccess, String idError) {
    button.click();
    waitAlert(idSuccess, idError);
  }

  /**
   * Closing a modal window by clicking window's icon
   *
   * @param icon closing modal window
   */
  public static void clickIconAndWaitHideModal(WebElement icon) {
    icon.click();
    wait.until(
    ExpectedConditions.not(
            ExpectedConditions.visibilityOfAllElements(Collections.singletonList(icon))));
  }

  /**
   * Closing a modal window by clicking some button
   *
   * @param button closing modal window
   */
  public static void clickButtonAndWaitHideModal(WebElement button) {
    WebElement closeIcon = new ModalClose_Page(new PageObject()).getCloseIcon();
    button.click();
    wait.until(
    ExpectedConditions.not(
            ExpectedConditions.visibilityOfAllElements(Collections.singletonList(closeIcon))));
  }

  /**
   * Scrolls to be the button visible
   *
   * prevent to "WebDriverException: Element is not clickable at point"
   *
   * Warning: works with Opera 70 only
   *          Firefox - MoveTargetOutOfBoundsException
   *
   * @param button clickable button
   */
  private static void clickableElementVisible(WebElement button) {
    Actions actionProvider = new Actions(Drivers.getDriver());
    // Performs mouse move action onto the element
    actionProvider.moveToElement(button).build().perform();
  }

  public static void scrollToElement(WebElement element){
    Actions actions = new Actions(Drivers.getDriver());
    actions.moveToElement(element);
    actions.perform();
  }

  public static void scrollToElement(String id){
    WebElement element = Drivers.getDriver().findElement(By.id(id));
    scrollToElement(element);
  }




}
