package pstsp.support.selenium_utils;

/**
 * Utilities for a table with subheaders
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import pstsp.support.test_utils.Fail;

import java.util.*;

public class SubTable {

  /**
   * Gets content of all subheaders
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return set of subheader's texts
   */
  public static Set<String> getAllSubHeaders(WebElement table, String idRowSubHeader_X) {
    Set<String> content = SubTable.getSubHeaders(table, idRowSubHeader_X, false);
    return content;
  }

  /**
   * Gets list of content of all subheaders
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return list of subheader's texts
   */
  public static List<String> getAllSubHeadersList(WebElement table, String idRowSubHeader_X) {
    List<String> content = SubTable.getSubHeadersList(table, idRowSubHeader_X, false);
    return content;
  }

  /**
   * Gets content of all "active" subheaders, eg. subheaders with following valid rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return List of active subheader's texts
   */
  public static Set<String> getActiveSubHeaders(WebElement table, String idRowSubHeader_X) {
    Set<String> content = SubTable.getSubHeaders(table, idRowSubHeader_X, true);
    return content;
  }

  /**
   * Gets list of content of all "active" subheaders, eg. subheaders with following valid rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return set of active subheader's texts
   */
  public static List<String> getActiveSubHeadersList(WebElement table, String idRowSubHeader_X) {
    List<String> content = SubTable.getSubHeadersList(table, idRowSubHeader_X, true);
    return content;
  }

  /**
   * Gets content of all "passive" subheaders, eg. subheaders with following none or empty rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return set of passive subheader's texts
   */
  public static Set<String> getPassiveSubHeaders(WebElement table, String idRowSubHeader_X) {
    Set<String> content = SubTable.getSubHeaders(table, idRowSubHeader_X, false);
    content.removeAll(SubTable.getSubHeaders(table, idRowSubHeader_X, true));
    return content;
  }

  /**
   * Gets list of content of all "passive" subheaders, eg. subheaders with following none or empty rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @return list of passive subheader's texts
   */
  public static List<String> getPassiveSubHeadersList(WebElement table, String idRowSubHeader_X) {
    List<String> content = SubTable.getSubHeadersList(table, idRowSubHeader_X, false);
    content.removeAll(SubTable.getSubHeadersList(table, idRowSubHeader_X, true));
    return content;
  }

  /**
   * Finds a row with the displayed text in the specified subheader and specified column
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @param searchedColumn number of one column (numbers start from 0)
   * @param columnText searched text in a column
   * @return WebElement row or null otherwise
   */
  public static WebElement getRowBySubHeaderAndColumnText(WebElement table, String idRowSubHeader_X,
                  String subHeaderText, int searchedColumn, String columnText) {
    List<WebElement> trs = getRowsBySubHeader(table, idRowSubHeader_X, subHeaderText);
    for (WebElement tr : trs) {
      String displayedText = Table.getOneRowContents(tr, searchedColumn);
      if (displayedText.equals(columnText) == true) {
        return tr;
      }
    }
    Fail.failTestDueToMissingTextInTable(table.getAttribute("id"),
            subHeaderText + ":" + columnText);
    return null;
  }

  /**
   * Finds a row with the displayed text1 and text2 in the specified subheader and specified columns
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @param searchedColumn1 number of one column (numbers start from 0)
   * @param columnText1 searched text in a column
   * @param searchedColumn2 number of one column (numbers start from 0)
   * @param columnText2 searched text in a column
   * @return WebElement row or null otherwise
   */
  public static WebElement getRowBySubHeaderAndColumnText(WebElement table, String idRowSubHeader_X,
                  String subHeaderText, int searchedColumn1, String columnText1,
                                        int searchedColumn2, String columnText2) {
    List<WebElement> trs = getRowsBySubHeader(table, idRowSubHeader_X, subHeaderText);
    for (WebElement tr : trs) {
      String displayedText1 = Table.getOneRowContents(tr, searchedColumn1);
      String displayedText2 = Table.getOneRowContents(tr, searchedColumn2);
      if (displayedText1.equals(columnText1) == true  &&
          displayedText2.equals(columnText2) == true ) {
        return tr;
      }
    }
    Fail.failTestDueToMissingTextInTable(table.getAttribute("id"),
            subHeaderText + ":" + columnText1 + " " + columnText2);
    return null;
  }

  /**
   * Finds the first row bellow the specified subheader and gets text in specified column in this row
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @param searchedColumn number of one column (numbers start from 0)
   * @return text in column or null otherwise
   */
  public static String getColumnTextBySubHeaderFirstRow(WebElement table, String idRowSubHeader_X,
                                                String subHeaderText, int searchedColumn) {
    WebElement tr = getRowsBySubHeader(table, idRowSubHeader_X, subHeaderText).get(0);
    List<WebElement> tds = tr.findElements(By.tagName("td"));
    if (tds.size() >= searchedColumn) {
      return Table.getOneRowContents(tr, searchedColumn);
    }
    else {
      Fail.failTestDueToMissingTextInTable(table.getAttribute("id"), subHeaderText);
      return null;
    }
  }

  /**
   * Finds all rows bellow the specified subheader and gets text in specified column in these rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @param searchedColumn number of one column (numbers start from 0)
   * @return set of texts in column or empty set otherwise
   */
  public static Set<String> getColumnTextsBySubHeader(WebElement table, String idRowSubHeader_X,
                                                 String subHeaderText, int searchedColumn) {
    Set<String> columnTexts = new HashSet<>();
    List<WebElement> trs = getRowsBySubHeader(table, idRowSubHeader_X, subHeaderText);
    for (WebElement tr : trs) {
      List<WebElement> tds = tr.findElements(By.tagName("td"));
      if (tds.size() >= searchedColumn) {
        columnTexts.add(Table.getOneRowContents(tr, searchedColumn));
      }
    }
    return columnTexts;
  }

  /**
   * Finds all rows bellow the specified subheader and gets whole texts in these rows
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @return set of texts in all columns or empty set otherwise
   */
  public static Set<String> getWholeRowTextBySubHeader(WebElement table, String idRowSubHeader_X,
                                                       String subHeaderText) {
    Set<String> rowsText = new TreeSet<>();
    List<WebElement> trSub = getRowsBySubHeader(table, idRowSubHeader_X, subHeaderText);
    for (WebElement tr : trSub) {
      rowsText.add(tr.getText());
    }
    return rowsText;
  }
   
  /**
   * Gets list of content of subheaders
   * 
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param activeOnly true=only active, false=all
   * @return list of subheader's texts
   */
  private static List<String> getSubHeadersList(WebElement table, String idRowSubHeader_X,
                                           boolean activeOnly) {
    List<String> content = new ArrayList<>();
    List<WebElement> ths = table.findElements(By.tagName("th"));
    for (WebElement th : ths) {
      String thId = th.getAttribute("id").trim();
      if (thId.startsWith(idRowSubHeader_X) == true) {
        String subjectName = th.getText().trim();
        WebElement tbody = th.findElement(By.xpath("../following::tbody"));
        WebElement tr = tbody.findElement(By.tagName("tr"));
        List<WebElement> tds = tr.findElements(By.tagName("td"));
        if (activeOnly == true) {
          if (tds.size() > 1) {
            content.add(subjectName);
          } 
        }
        else {
          content.add(subjectName);
        }
      }
    }
    return content;
  }

  /**
   * Gets content of subheaders
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param activeOnly true=only active, false=all
   * @return set of subheader's texts
   */
  private static Set<String> getSubHeaders(WebElement table, String idRowSubHeader_X,
                                           boolean activeOnly) {
    Set<String> content = new TreeSet<>();
    content.addAll(getSubHeadersList(table, idRowSubHeader_X, activeOnly));
    return content;
  }

  /**
   * Gets list of all rows bellow specified header
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param subHeaderText displayed text in a subheader
   * @return list of row's web elements or empty list otherwise
   */
  private static List<WebElement> getRowsBySubHeader(WebElement table, String idRowSubHeader_X,
                                                     String subHeaderText) {
    List<WebElement> trsSub = new ArrayList<>();
    List<WebElement> ths = table.findElements(By.tagName("th"));
    for (WebElement th : ths) {
      String thId = th.getAttribute("id").trim();
      if (thId.startsWith(idRowSubHeader_X) == true) {
        String subjectName = th.getText().trim();
        if (subjectName.contains(subHeaderText) == false) {
          continue;
        }
        WebElement tbody = th.findElement(By.xpath("../following::tbody"));
        trsSub = tbody.findElements(By.tagName("tr"));
        break;
      }
    }
    return trsSub;
  }

  /**
   * Gets number of max. participants for the specified exam date and the specified subject
   *
   * @param table WebElement table
   * @param idRowSubHeader_X beginning (constant part) of ID of subheader's row
   * @param examDate
   * @param subjectName
   * @return number of max. participants at exam date and subject
   */
  public static int getMaxNumberOfParticipants(WebElement table, String idRowSubHeader_X,
                                               String examDate, String subjectName) {
    Set<String> rows = getWholeRowTextBySubHeader(table, idRowSubHeader_X, subjectName);
    String number = "0";
    for (String row : rows) {
      if (row.contains(examDate) == true) {
        int begin = row.indexOf("/");
        int end = row.indexOf(")");
        number = row.substring(begin + 1, end);
        break;
      }
    }
    return Integer.parseInt(number);
  }

}
