package pstsp.support.selenium_utils;

/**
 * Utilities for an ordinary table
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import pstsp.support.test_utils.Fail;

import java.util.*;

public class Table {

  /**
   * Gets texts in the table's header columns
   * 
   * @param table
   * @return displayed texts in columns
   */
  public static String[] getHeaderColumnsText(WebElement table) {
    WebElement theadTr = table.findElements(By.tagName("tr")).get(0);
    List<WebElement> ths = theadTr.findElements(By.tagName("th"));
    String[] displayed = new String[ths.size()];
    int i = 0;
    for (WebElement th : ths) {
      displayed[i] = th.getText().trim();
      i++;
    }
    return displayed;
  }

  /**
   * Get table's whole text (eg. each row's columns)
   * including header's columns
   *  
   * @param table
   * @return set of row's texts
   */
  public static List<String> getAllRowsWholeText(WebElement table) {
    List<String> content = new ArrayList<>();
    List<WebElement> trs = table.findElements(By.tagName("tr"));
    for (WebElement tr : trs) {
      String text = tr.getText().trim();
      content.add(text);
    }
    return content;
  }

  /**
   * Get table's whole text (eg. each row's columns)
   * including header's columns
   *
   * @param table
   * @return set of row's elements
   */
  public static List<List<WebElement>> getTableContent(WebElement table) {

    List<WebElement> trs = table.findElements(By.tagName("tr"));
    List<List<WebElement>> rows = new ArrayList<>();
    for (WebElement tr : trs) {
      List<WebElement> tds = tr.findElements(By.tagName("td"));
      List<WebElement> row = new ArrayList<>();
      for(WebElement td : tds) {
        row.add(td);
      }
      rows.add(row);

    }
    return rows;
  }



  /**
   * Gets text in the one or several columns in one specified row
   * 
   * @param table
   * @param idRow_X beginning (constant part) of ID of row
   * @param validColumns list of specified columns (numbers start from 0), eg. second and fourth column will be 1,3
   * @return List of column's texts
   */
  public static List<String> getColumnsTextList(WebElement table, String idRow_X, int... validColumns) {
    List<String> content = new ArrayList<>();
    List<WebElement> trs = table.findElements(By.tagName("tr"));
    for (WebElement row : trs) {
      String rowId = row.getAttribute("id").trim();
      if (rowId.startsWith(idRow_X) == true) {
        String text = Table.getOneRowContents(row, validColumns);
        if (text.isEmpty() == false) {
          content.add(text);
        }
      }
    }
    return content;
  }

  /**
   * Gets text in the one or several columns in one specified row
   *
   * @param table
   * @param idRow_X beginning (constant part) of ID of row
   * @param validColumns list of specified columns (numbers start from 0), eg. second and fourth column will be 1,3
   * @return set of column's texts
   */
  public static Set<String> getColumnsText(WebElement table, String idRow_X, int... validColumns) {
    List<String> list = getColumnsTextList(table, idRow_X, validColumns);
    Set<String> content = new TreeSet<>(list);
    return content;
  }

  /**
   * Finds a row with the displayed text and gets texts from specified columns in this row
   *  
   * @param table
   * @param displayedText text displayed in any column in one row
   * @param validColumns list of specified columns (numbers start from 0), eg. second and fourth column will be 1,3
   * @return set of column's texts
   */
  public static String getTextInAnotherColumns(WebElement table, String displayedText, int... validColumns) {
    String content = null;
    List<WebElement> tds = table.findElements(By.tagName("td"));
    for (WebElement td : tds) {
      String column = td.getText().trim();
      if (column.equals(displayedText) == true) {
        WebElement row = td.findElement(By.xpath("parent::tr"));
        content = Table.getOneRowContents(row, validColumns);
        break;
      }
    }
    return content;
  }

  /**
   * Finds a row with the displayed text in the specified column
   * 
   * @param table
   * @param searchedColumn number of one column (numbers start from 0)
   * @param columnText text displayed in any column in one row
   * @return WebElement row or null otherwise
   */
  public static WebElement getRowWithTextInColumn(WebElement table, int searchedColumn, String columnText) {
    List<WebElement> trs = table.findElements(By.tagName("tr"));
    for (WebElement row : trs) {
      String displayedText = Table.getOneRowContents(row, searchedColumn);
      if (displayedText.contains(columnText) == true) {
        return row;
      }
    }
    String id = table.getAttribute("id").trim();
    Fail.failTestDueToMissingTextInTable(id, columnText);
    return null;    
  }

  /**
   * Gets texts from specified columns in one row
   * test are separated by semicolon ";", excluding the last one (eg. "first;second;third")
   *  
   * @param row specified row
   * @param validColumns list of specified columns (numbers start from 0), eg. second and fourth column will be 1,3
   * @return column's text
   */
  public static String getOneRowContents(WebElement row, int... validColumns) {
    List<WebElement> tds = row.findElements(By.tagName("td"));
    if (tds.isEmpty() == true) {
      return "";
    }
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < validColumns.length; i++) {
      int j = validColumns[i];
      if (j >= tds.size()) {
        continue;
      }
      WebElement td = tds.get(j);
      sb.append(td.getText().trim());
      sb.append(";");
    }
    if (sb.lastIndexOf(";") >= 0) {
      sb.deleteCharAt(sb.lastIndexOf(";"));
    }
    String text = sb.toString();
    return text;
  }

  /*
   * Return number (order) of column
   */
  public static int getColumnNumber(String[] tableHeaders, String headerName) {
    for (int i = 0; i < tableHeaders.length; i++) {
      if (tableHeaders[i].equals(headerName)) {
        return i;
      }
    }
    String headers = Arrays.toString(tableHeaders);
    Fail.failTestDueToMissingTextInTable(headers, headerName);
    return -1;
  }

  /**
   * Transform string with several items separated by ", "
   * into TreeSet
   * If input string is empty, than returns empty Set.
   *
   * @param items
   * @return set o items
   */
  public static Set<String> convertStringToSet(String items) {
    if(items.equals("")){
      return new TreeSet<>();
    }
    String[] values = items.split(", ");
    Set<String> set = new TreeSet<>(Arrays.asList(values));
    return set;
  }
}
