package pstsp.support.selenium_utils;

/**
 * Low level utils for web elements
 * and failing tests
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.Select;
import pstsp.support.basic.Const;
import pstsp.support.test_utils.Fail;
import pstsp.support.tools.Drivers;

import java.util.*;

public class Utils {
  /**
   * Returns a current URL
   * @return current URL
   */
  public static String getCurrentUrl() {
    return Drivers.getDriver().getCurrentUrl();
  }

  /**
   * Find a button by its ID in another "parent" web element (mostly in table row)
   * all buttons in the table rows start with the same ID and differ at their ends with row numbers only
   * due to ID is not complete, but it is just fixed (common) first part
   *
   * if is not successful fails test by Fail.failTestDueToMissingElement()
   *
   * @param element parent web element
   * @param idButton_X first part of ID
   * @return button web element or null
   */
  public static WebElement getButtonInWebElement(WebElement element, String idButton_X) {
    List<WebElement> buttons = element.findElements(By.tagName("button"));
    for (WebElement button : buttons) {
      String idButton = button.getAttribute("id");
      if (idButton.startsWith(idButton_X) == true) {
        return button;
      }
    }
    Fail.failTestDueToMissingElement(idButton_X);
    return null;
  }

  /**
   * Find an element by its tag name and ID_X
   * all elements start with the same ID and differ at their ends with numbers only
   * due to ID is not complete, but it is just fixed (common) first part
   * moreover element must be visible, eg. style="", not style="display: none;"
   *
   * if is not successful fails test by Fail.failTestDueToMissingElement()
   *
   * @param tagName tag name of a web element
   * @param id_X first part of ID
   * @return web element or null
   */
  public static WebElement getElementByTagName(String tagName, String id_X) {
    List<WebElement> elements = Drivers.getDriver().findElements(By.tagName(tagName));
    for (WebElement element : elements) {
      String idElement = element.getAttribute("id");
      String styleElement = element.getAttribute("style");
      if (idElement.startsWith(id_X) == true  &&  styleElement.isEmpty()) {
        return element;
      }
    }
    Fail.failTestDueToMissingElement(id_X);
    return null;
  }

  /**
   * Find a selection by its ID in another "parent" web element (mostly in table row)
   * all selection in the table rows start with the same ID and differ at their ends with row numbers only
   * due to ID is not complete, but it is just fixed (commnon) first part
   *
   * if is not successful fails test by Fail.failTestDueToMissingElement()
   *
   * @param element parent web element
   * @param idSelection_X first part of ID
   * @return selection web element or null
   */
  public static WebElement getSelectionInWebElement(WebElement element, String idSelection_X) {
    List<WebElement> selections = element.findElements(By.tagName("select"));
    for (WebElement select : selections) {
      String idButton = select.getAttribute("id");
      if (idButton.startsWith(idSelection_X) == true) {
        return select;
      }
    }
    Fail.failTestDueToMissingElement(idSelection_X);
    return null;
  }

  /**
   * Activate (choose) new selection option according to its text
   *
   * if is not successful fails test by Fail.failTestDueToMissingTextInTable()
   *
   * @param selection web element selection
   * @param text text on desired option
   */
  public static void setSelection(WebElement selection, String text) {
    List<WebElement> options = selection.findElements(By.tagName("option"));
    for (WebElement option : options) {
      String textOption = removeDuplicateWhiteSpaces(option.getText());
      if (textOption.startsWith(text)) {
        option.click();
        return;
      }
    }
    String id = selection.getAttribute("id");
    Fail.failTestDueToMissingTextInTable(id, text);
  }

  public static List<String> getFormErrorText(){
    List<WebElement> error = Drivers.getDriver().findElements(By.className(Const.ERROR_CLASS));

    if(error.isEmpty()) return new ArrayList<>();

    List<String> result = new ArrayList<>();
    error.forEach(a -> result.add(a.getText().strip()));
    return result;
  }

  /**
   * Texts in HTML format can contain plenty of white spaces
   * this method removes duplicate ones
   *
   * @param text with possible duplicate white spaces
   * @return text without duplicate white spaces
   */
  public static String removeDuplicateWhiteSpaces(String text) {
    String s = text.trim();
    s = s.replaceAll("\\p{Space}", " ");
    while (s.contains("  ") == true) {
      s = s.replaceAll("  ", " ");
    }
    return s;
  }

  /**
   * Returns all selection option texts
   *
   * @param selection web element selection
   * @return all texts in whole selection
   */
  public static List<String> getSelectionContentList(WebElement selection) {
    List<String> content = new ArrayList<>();
    List<WebElement> options = selection.findElements(By.tagName("option"));
    for (WebElement option : options) {
      String style = option.getAttribute("style");
      if (style != null && style.equals("display: none;") == true) {
        continue;
      }
      content.add(removeDuplicateWhiteSpaces(option.getText()));
    }
    return content;
  }

  /**
   * Returns all selection option texts
   *
   * @param selection web element selection
   * @return all texts in whole selection
   */
  public static Set<String> getSelectionContent(WebElement selection) {
    Set<String> content = new TreeSet<>();
    content.addAll(getSelectionContentList(selection));
    return content;
  }

  /**
   * Returns selected option's text
   *
   * @param selection web element selection
   * @return text of selected option
   */
  public static String getSelectedOption(WebElement selection) {
    Select select = new Select(selection);
    String text = select.getAllSelectedOptions().get(0).getText();
    return text;
  }

  /**
   * Set (switch on or switch off) of a check box
   *
   * @param checkBox web element
   * @param set true for switch on or false for switch off
   */
  public static void setCheckBox(WebElement checkBox, boolean set) {
    if (checkBox.isSelected() != set) {
      checkBox.sendKeys(Keys.SPACE);
//      element.click();  // does not work
    }
  }

  /**
   * Checks if the set (usually content rows of a table or list of selection's options)
   * contains the expected text
   *
   * @param set set of all texts
   * @param expectedText
   * @return true if contains, false otherwise
   */
  public static boolean isSetContainsString(Set<String> set, String expectedText) {
    boolean result = false;
    for (String item : set) {
      if (item.contains(expectedText) == true) {
        result = true;
        break;
      }
    }
    return result;
  }



  /**
   * Checks if the menu item is active/selected, eg. has attribute
   * class="nav-link active"
   *
   * @param menuItem menu item
   * @return true if selected, false otherwise
   */
  public static boolean isMenuItemSelected(WebElement menuItem) {
    String classAttr = menuItem.getAttribute("class");
    return classAttr.endsWith("active");
  }

  /**
   * Find web element addressed by tagName
   * it has special use in a complex tables
   *
   * @param tagName tag name
   * @return ID
   */
  public static WebElement getElementByTagName(String tagName) {
    List<WebElement> elements = Drivers.getDriver().findElements(By.tagName(tagName));
    for (WebElement element : elements) {
      if (element.getAttribute("style").isEmpty() == true) {
        return element;
      }
    }
    Fail.failTestDueToMissingElement("required tagName of <" + tagName + ">");
    return null;
  }

  /**
   * Prepares long string containing digits only
   * for negative tests purpose
   *
   * @param numberOfDigits length od string
   * @return string of digits
   */
  public static String getBigNumber(int numberOfDigits) {
    StringBuilder sb = new StringBuilder(numberOfDigits);
    Random rand = new Random(0);
    for (int i = 0; i < numberOfDigits; i++) {
      sb.append((char) ('0' + rand.nextInt(10)));
    }
    return sb.toString();
  }

  /**
   * Prepares long string containing ASCII chars
   * for negative tests purpose
   *
   * @param numberOfChars length od string
   * @return string of chars
   */
  public static String getBigString(int numberOfChars) {
    StringBuilder sb = new StringBuilder(numberOfChars);
    Random rand = new Random(0);
    for (int i = 0; i < numberOfChars; i++) {
      int j = rand.nextInt(95);
      sb.append((char)(' ' + j));
    }
    return sb.toString();
  }

  /**
   * Prepares long string containing chars which ends like an email
   * for negative tests purpose
   *
   * @param numberOfChars length od string
   * @return string of chars
   */
  public static String getBigEmail(int numberOfChars) {
    StringBuilder sb = new StringBuilder(numberOfChars);
    sb.append("xy@ab.c");
    int lengthOfBegin = sb.length();
    Random rand = new Random(0);
    for (int i = 0; i < numberOfChars - lengthOfBegin; i++) {
      int j = rand.nextInt(26);
      sb.append((char)('a' + j));
    }
    return sb.toString();
  }

  /**
   * Get list of subject names sorted alphabetically
   *
   * @param subjects collection of strings, where subject name is the very first item
   *                  "Computation Structures;Thomas Scatterbrained;5" or
   *                  "Computation Structures"
   * @return list of sorted subjects' names
   */
  public static List<String> sortSubjectNamesAlphabetically(Collection<String> subjects) {
    List<String> names = new ArrayList<>();
    for (String subject : subjects) {
      String subjectName = subject;
      int index = subject.indexOf(";");
      if (index > 0) {
        subjectName = subject.substring(0, index);
      }
      names.add(subjectName);
    }
    List<String> subjectsNames = Utils.sortAlphabetically(names);
    return subjectsNames;
  }

  /**
   * Sorts collection of strings alphabetically
   *
   * @param colection collection of strings
   * @return list of sorted strings
   */
  public static List<String> sortAlphabetically(Collection<String> colection) {
    TreeSet<String> set = new TreeSet<>(String::compareToIgnoreCase);
    set.addAll(colection);
    List<String> list = new ArrayList<>(set);
    return list;
  }

  /**
   * Sorts collection of full names alphabetically by last name
   *
   * @param fullNames collection of full names
   * @return list of sorted full names
   */
  public static List<String> sortFullNamesAlphabetically(Collection<String> fullNames) {
    List<String> list = swapNames(fullNames);
    TreeSet<String> set = new TreeSet<>(String::compareToIgnoreCase);
    set.addAll(list);
    List<String> sortedList = new ArrayList<>(set);
    return swapNames(sortedList);
  }

  /**
   * Swap First name and Last name
   *  "Bart Simpson" -> "Simpson Bart"
   *
   * @param fullNames collection of full names
   * @return list of full names with swapped first an last name
   */
  private static List<String> swapNames(Collection<String> fullNames) {
    List<String> list = new ArrayList<>();
    for (String fullName : fullNames) {
      int index = fullName.lastIndexOf(" ");
      if (index >= 0) {
        String name = fullName.substring(index + 1) + " "
                + fullName.substring(0, index);
        list.add(name);
      }
      else {
        // empty
        list.add(fullName);
      }
    }
    return list;
  }
}
