package pstsp.support.services.entities.storage;

import org.openqa.selenium.By;
import org.openqa.selenium.InvalidArgumentException;
import org.openqa.selenium.WebElement;
import pstsp.support.pageobjects.project.Project_Page;
import pstsp.support.selenium_utils.Click;
import pstsp.support.selenium_utils.MyAlert;
import pstsp.support.services.entities.ProjectTableRow;
import pstsp.support.tools.Drivers;

import java.util.List;
/**
 * Project page table
 * @author David Kuta
 */
public class ProjectTable {

    public static final int MIN_OPTION = 0;
    public static final int MAX_OPTION = 3;

    // color options
    public static final String NOT_DONE = "danger";
    public static final String SEMI_DONE = "warning";
    public static final String DONE = "success";
    public static final String NOT_ASSIGNED = "secondary";

    //prefixes
    public static final String BTN_PREFIX = "btn-";
    public static final String BG_PREFIX = "bg-";


    private final List<ProjectTableRow> rowList;

    private final String tableType;

    private final String userRole;

    public ProjectTable(List<ProjectTableRow> rowList, String tableType, String userRole) {
        this.rowList = rowList;
        this.tableType = tableType;
        this.userRole = userRole;
    }

    /**
     * sets value and waits for alert
     * @param row row in table
     * @param option option, number from 0 to 3
     */
    public void setStateValue(int row, int option) {
        if(row >= this.getRowCount()){
            throw new InvalidArgumentException("Row " + row +" out of range from maximum value " +this.getRowCount());
        }

        if(option > MAX_OPTION){
            throw new InvalidArgumentException("Option " + option +" out of range from maximum value " + MAX_OPTION);
        }
        Click.scrollToElement(this.rowList.get(row).getChangeButtonToggle());
        //Click.clickAndWaitBy(this.rowList.get(row).getChangeButtonToggle(), By.id("projectPage-usedExp-leader-table-" + 3 + "-0"));
        String optionId = getOptionIdPrefixByUserRole(this.userRole, row,option);
        Click.clickAndWaitBy(this.rowList.get(row).getChangeButtonToggle(), By.id(optionId));
        WebElement optionToSelect = Drivers.getDriver().findElement(By.id(optionId));
        Click.clickAndWaitId(optionToSelect, MyAlert.MY_ALERT_ID);
    }

    private String getOptionIdPrefixByUserRole(String role, int row, int option) {
        String prefix = "";
        prefix = tableType + "-" + role + "-" + row + "-" + option;

        return prefix;
    }

    /**
     *
     * @param stepName name of step
     * @return index in row
     */
    public int getIndexForStepName(String stepName){
        for (int i = 0; i < rowList.size(); i++) {
            if(rowList.get(i).getStepName().equals(stepName)){
                return i;
            }
        }
        throw new IllegalStateException("Invalid step in table with text: " + stepName);
    }

    /**
     *
     * @param stepName name of step
     * @param otherTextInfo other text info, used only in used experience table
     * @return index
     */
    public int getIndexForStepNameAndTextInfo(String stepName, String otherTextInfo){
        for (int i = 0; i < rowList.size(); i++) {
            if(rowList.get(i).getStepName().equals(stepName) && rowList.get(i).getOtherTextInfo().equals(otherTextInfo)){
                return i;
            }
        }
        throw new IllegalStateException("Invalid step in table with text: " + stepName + " and step info: " + otherTextInfo);
    }

    public List<ProjectTableRow> getRowList() {
        return rowList;
    }

    public String getTableType() {
        return tableType;
    }

    public String getUserRole() {
        return userRole;
    }

    public int getRowCount(){
        return rowList.size();
    }

    public String getActualValueEnabled(int row){
        return this.rowList.get(row).getChangeButtonColor();
    }

    public String getActualValueDisabled(int row){
        return this.rowList.get(row).getOtherButtonColor();
    }

    /**
     * Gets count of danger steps
     */
    public int getDangerCount(){
        int cnt = 0;
        for (ProjectTableRow row: rowList) {
            if(row.getChangeButtonColor().equals("danger")){
                cnt++;
            }

        }
        return cnt;
    }

    /**
     * Gets count of warning steps
     */
    public int getWarningCount(){
        int cnt = 0;
        for (ProjectTableRow row: rowList) {
            if(row.getChangeButtonColor().equals("warning")){
                cnt++;
            }

        }
        return cnt;
    }

    /**
     * Gets count of success steps
     */
    public int getSuccessCount(){
        int cnt = 0;
        for (ProjectTableRow row: rowList) {
            if(row.getChangeButtonColor().equals("success")){
                cnt++;
            }

        }
        return cnt;
    }

    public String getColorForChoice(int choice){
        switch (choice){
            case 0:{
                return NOT_DONE;
            }
            case 1:{
                return SEMI_DONE;
            }
            case 2:{
                return DONE;
            }
            case 3:{
                return NOT_ASSIGNED;
            }
            default:{
                throw new IllegalArgumentException("Choice out of range. Correct is from <0;3>");
            }
        }
    }
}
