package pstsp.support.test_oracle.storage;

/**
 * Class for reading and holding of all existing users
 * according to configuration file 'users_list.txt'
 *
 * The class is a singleton
 *
 * @author Pavel Herout
 * @version 2022-01-25
 */


import pstsp.support.test_oracle.OracleInicialization;
import pstsp.support.test_oracle.entities.User;
import pstsp.support.test_utils.Fail;
import pstsp.support.tools.ProjectLogger;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

public class UsersSet {

  /** Only one instance - Singleton design pattern */
  private static final UsersSet INSTANCE = new UsersSet();

  /** all users */
  private Set<User> users;

  /** Private constructor - Singleton design pattern */
  private UsersSet() {
    users = new TreeSet<User>();
  }

  /**
   * Get only one instance
   *
   * @return only one instance
   */
  public static UsersSet getInstance() {
    return INSTANCE;
  }    

  /**
   * Get all users
   *
   * @return all users
   */
  public Set<User> getAllUsers() {
    return users;
  }

  /**
   * Delete all users
   */
  public void restore() {
    users.clear();
  }

  /**
   * Get user for specified user name
   *
   * @param userName user's user name
   * @return user or fail test
   */
  public User getUserByUsername(String userName) {
    for (User user : getInstance().users) {
      if (user.getUserName().equals(userName)) {
        return user;
      }
    }
    Fail.failTestDueToMissingObjectInOracleStorage("UsersSet", userName);
    return null;
  }

  /**
   * Get user for specified full name
   *
   * @param fullName user's full name
   * @return user or fail test
   */
  public User getUserByFullname(String fullName) {
    for (User user : getInstance().users) {
      if (user.getFullName().equals(fullName)) {
        return user;
      }
    }
    Fail.failTestDueToMissingObjectInOracleStorage("UsersSet", fullName);
    return null;
  }

  /**
   * Get user for specified last name
   * the most often used method from all tests
   *
   * @param lastName user's last name
   * @return user or fail test
   */
  public User getUserByLastName(String lastName) {
    for (User user : users) {
      if (user.getLastName().equals(lastName)) {
        return user;
      }
    }
    Fail.failTestDueToMissingObjectInOracleStorage("UsersSet", lastName);
    return null;
  }

  /**
   * Read all users from configuration file 'users_list.txt'
   * all activities are logged
   */
  public void readUsers() {
    List<String> lines = OracleInicialization.readValidLinesFile(
            OracleInicialization.USERS_FILE_URI);
    if (lines.isEmpty() == true) {
      ProjectLogger.logger.error("No valid user in the file");
      throw new IllegalArgumentException("No valid user in the file");
    }

    for (String line : lines) {
      String[] parts = line.split(";");
      if (parts.length < 2) {
        String message = "There is a missing part of a user name in the line '"
                + line + "' in file "
                + OracleInicialization.USERS_FILE_URI;
        ProjectLogger.logger.error(message);
        throw new IllegalArgumentException("Missing part of a user name");
      }
      String[] jmeno = parts[1].split(" ");
      if (jmeno.length < 2) {
        String message = "There is a missing part of a user full name in the line '"
                + line + "' in file "
                + OracleInicialization.USERS_FILE_URI;
        ProjectLogger.logger.error(message);
        throw new IllegalArgumentException("Missing part of a full name");
      }

      User user = new User(parts[0], jmeno[0], jmeno[1]);
      users.add(user);
      ProjectLogger.logger.trace("User:             " + user);
    }
  }
}
