package pstsp.support.test_utils;

/**
 * Class for wrapping assertions
 * Two main reasons for usage:
 * 1) add error message (usually with ID) into default error message
 * 2) count real number of tests, eg. one test = one assertion
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */


import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import pstsp.support.selenium_utils.MyAlert;
import pstsp.support.selenium_utils.Utils;
import pstsp.support.tools.Drivers;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import static org.junit.jupiter.api.Assertions.*;

public class Check {
  /**
   * Do not use it often, because URL can be extended about some parameters
   *
   * @param expectedUrl
   */
  public static void checkActualUrl(String expectedUrl) {
    TestEvaluation.incrementNumberOfAsserts();
    String url = Utils.getCurrentUrl();
    String message = "The actual URL is wrong.";
    assertEquals(expectedUrl, url, message);
  }

  /**
   * Use only after successful logging, where Student or Teacher Overview apperars
   *
   * @param expectedUrl_1
   * @param expectedUrl_2
   */
  public static void checkActualUrl(String expectedUrl_1, String expectedUrl_2) {
    TestEvaluation.incrementNumberOfAsserts();
    String url = Utils.getCurrentUrl();
    String message = "The actual URL is wrong.";
    if (url.equals(expectedUrl_2) == false) {
      assertEquals(expectedUrl_1, url, message);
    }
    else {
      assertEquals(expectedUrl_2, url, message);
    }
  }


  public static boolean checkText(String expected, String actual, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' has the wrong text.";
    assertEquals(expected, actual, message);
    return (expected.equals(actual) ? true : false);
  }

  public static void checkTooltip(String expected, String actual, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' has the wrong tooltip.";
    assertEquals(expected, actual, message);
  }

  public static void checkPlaceholder(String expected, String actual, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' has the wrong placeholder.";
    assertEquals(expected, actual, message);
  }

  public static void checkTextEmpty(String actual, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not empty.";
    assertTrue(actual.isEmpty(), message);
  }

  public static void checkCorrectness(boolean correct, String id, String reason) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not correct. Reason: " + reason;
    assertTrue(correct, message);
  }

  public static void checkIsEnabled(boolean enabled, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not enabled.";
    assertTrue(enabled, message);
  }

  public static void checkIsReadOnly(boolean readOnly, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not read only.";
    assertTrue(readOnly, message);
  }

  public static void checkIsVisible(boolean visible, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not visible.";
    assertTrue(visible, message);
  }

  public static void checkBoolean(boolean expected, boolean actual, String id) {
    TestEvaluation.incrementNumberOfAsserts();
    String message = "Element ID='" + id + "' is not " + expected + "but, " + actual + "instead.";
    assertEquals(expected,actual,message);
  }

  /**
   * Checks contents of two arrays of String
   * usually content of table headers' manes
   *
   * @param actualContent actual content
   * @param expectedContent expected content usually from page object
   * @param message error message
   */
  public static void checkContentsOfTwoArrays(String[] actualContent,
                                              String[] expectedContent, String message) {
    TestEvaluation.incrementNumberOfAsserts();
    String errMessage = message +
            ": Actual content differs from expected content";
    assertArrayEquals(expectedContent, actualContent, errMessage);
  }

  /**
   * Checks contents of two list of String
   *
   * @param actualContent actual content
   * @param expectedContent expected content usually from page object
   * @param message error message
   */
  public static void checkContentsOfTwoLists(List<String> actualContent,
                           List<String> expectedContent, String message) {
    TestEvaluation.incrementNumberOfAsserts();
    String errMessage = message +
            ": Actual content differs from expected content";
    assertIterableEquals(expectedContent, actualContent, message);
  }

  /**
   * Symmetrically checks contents of two sets
   * usually content of table rows or selection's options
   *
   * @param actualContent actual content
   * @param expectedContent expected content usually from business logic
   * @param message error message
   */
  public static void checkContentsOfTwoSets(Set<String> actualContent,
                                            Set<String> expectedContent, String message) {
    TestEvaluation.incrementNumberOfAsserts();
    Set<String> symetricDifference = new TreeSet<>(expectedContent);
    symetricDifference.addAll(actualContent);
    Set<String> tmp = new TreeSet<>(expectedContent);
    tmp.retainAll(actualContent);
    symetricDifference.removeAll(tmp);
    Set<String> diffWithNewLine = new TreeSet<>();
    for (String row : symetricDifference) {
      String rowWithNewLine = row + "\n";
      diffWithNewLine.add(rowWithNewLine);
    }

    int errors = symetricDifference.size();
    String errMessage = message +
            ": Actual content differs from expected content in " + errors + " items:\n"
            + diffWithNewLine;
    assertEquals(0, errors, errMessage);
  }

    public static void checkPage(final String PAGE_ID){
      List<WebElement> elements = Drivers.getDriver().findElements(By.id(PAGE_ID));
      if (elements.size() == 1) {
        return;
      }
      Fail.failTestDueToMissingElement("Wrong page, expected element with id " + PAGE_ID + ">");
    }

    public static void checkAndCloseAlert(boolean success){
      String alertHeader = MyAlert.getAlert().getHeaderText();
      Check.checkText(success ? MyAlert.SUCCES_HEADER : MyAlert.FAIL_HEADER, alertHeader, MyAlert.MY_ALERT_ID);
      MyAlert.getAlert().closeAlert();
    }
}
