package pstsp.support.test_utils;

/**
 * Utils for reporting of failing tests
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.junit.jupiter.api.Assertions;

public class Fail {

  /**
   * Fails test in the case of missing web element
   * mostly used from PageObject.getElement()
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param id ID of missing web element
   */
  public static void failTestDueToMissingElement(String id) {
    String failMessage = "WebElement ID='"
            + id + "' has not been found";
    Assertions.fail(failMessage);
  }

  /**
   * Fails test in the case of missing web element
   * with specific text
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param text ID of missing web element
   */
  public static void failTestDueToMissingElementText(String text) {
    String failMessage = "WebElement with text='"
            + text + "' has not been found";
    Assertions.fail(failMessage);
  }

  /**
   * Fails test in the case of missing text on the web page
   * mostly used from Table
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param tableID ID of web element (mostly table), where text is searched
   * @param text searched text
   */
  public static void failTestDueToMissingTextInTable(String tableID, String text) {
    String failMessage = "Text='"
            + text + "' in the table ID='" + tableID + "' has not been found";
    Assertions.fail(failMessage);
  }

  /**
   * Fails test in the case of missing elementText in form
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param formId ID of web element, where element is searched
   * @param elementText searched text
   */
  public static void failTestDueToMissingTextInForm(String formId, String elementText) {
    String failMessage = "Text='"
            + elementText + "' in the table ID='" + formId + "' has not been found";
    Assertions.fail(failMessage);
  }

  /**
   * Fails test in the case of missing object in database
   * used by classes from support.test_oracle package
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param where where object 'what' is searched
   * @param what searched object
   */
  public static void failTestDueToMissingObjectInOracleStorage(Object where, Object what) {
    String failMessage = "Object='"
            + what + "' in the object='" + where + "' has not been found";
    Assertions.fail(failMessage);
  }

  /**
   * Fails test in the case of inconsistency in database
   * used by classes from support.test_oracle package
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param reason searched object
   */
  public static void failTestDueToProblemInOracleStorage(String reason) {
    Assertions.fail(reason);
  }

  /**
   * Fails test in the case of wrong value
   * used by classes from support.test_oracle package
   *
   * failMessage will be logged in TestEvaluation.testFailed()
   *
   * @param id id of surrounding element
   * @param wrongValue wrong value
   */
  public static void failTestDueToWrongValue(String id, String wrongValue) {
    String failMessage = "WebElement ID='"
            + id + "' has a wrong value='" + wrongValue + "'";
    Assertions.fail(failMessage);
  }
}
