package pstsp.support.test_utils;

/**
 * Evaluates result of any running test
 * <p>
 * MUST be activated in ANY testing file by:
 *
 * @code{@ExtendWith(TestEvaluation.class) public class NameOfClassTest }
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.api.extension.TestWatcher;
import pstsp.support.basic.Const;
import pstsp.support.tools.ProjectLogger;

import java.util.Optional;

public class TestEvaluation implements TestWatcher {
  /**
   * number of passed test
   */
  private static int testPassed = 0;

  /**
   * number of all test's failures
   */
  private static int testFailed = 0;

  /**
   * number of disabled tests
   */
  private static int testDisabled = 0;

  /**
   * number of aborted tests
   */
  private static int testAborted = 0;

  /**
   * number of asserts
   * it is useful for parametrized tests or active tests,
   * where several asserts in one test exist
   */
  private static int numberOfAsserts = 0;

  /**
   * cleans all counters
   * it is important in the case of repeated run of test suite
   */
  public static void clearCounters() {
    testPassed = 0;
    testFailed = 0;
    testDisabled = 0;
    testAborted = 0;
    numberOfAsserts = 0;
  }

  /**
   * Gets the number to ProjectLogger.logEndInfo()
   *
   * @return number of tests passed
   */
  public static int getTestPassed() {
    return testPassed;
  }

  /**
   * Gets the number to ProjectLogger.logEndInfo()
   *
   * @return number of tests failed
   */
  public static int getTestFailed() {
    return testFailed;
  }

  /**
   * Gets the number to ProjectLogger.logEndInfo()
   *
   * @return number of tests disabled
   */
  public static int getTestDisabled() {
    return testDisabled;
  }

  /**
   * Gets the number to ProjectLogger.logEndInfo()
   *
   * @return number of tests aborted
   */
  public static int getTestAborted() {
    return testAborted;
  }

  /**
   * Increments number of asserts
   * <p>
   * It is used in Check class only
   *
   * @return number of asserts
   */
  public static void incrementNumberOfAsserts() {
    numberOfAsserts++;
  }

  /**
   * Gets the number to ProjectLogger.logEndInfo()
   *
   * @return number of asserts
   */
  public static int getNumberOfAsserts() {
    return numberOfAsserts;
  }

  /**
   * Activity, when test has been aborted
   */
  @Override
  public void testAborted(ExtensionContext ec, Throwable throwable) {
    testAborted++;
    String message = "         " + Const.ERROR_LABEL + "test has been ABORTED";
    ProjectLogger.logger.error(message);
  }

  /**
   * Activity, when test is disabled
   */
  @Override
  public void testDisabled(ExtensionContext ec, Optional<String> optional) {
    testDisabled++;
  }

  /**
   * Activity, when test fails
   */
  @Override
  public void testFailed(ExtensionContext ec, Throwable throwable) {
    testFailed++;
    String testName = ec.getDisplayName();
    testName = testName.trim();
    String scr_message = "";
    String message = "         " + Const.ERROR_LABEL
            + getExceptionMessage(throwable.getMessage()) + scr_message;
    ProjectLogger.logger.error(message);
  }

  /**
   * Activity, when test pass
   */
  @Override
  public void testSuccessful(ExtensionContext ec) {
    testPassed++;
    // the name of the test is logged before the test begins
  }

  /**
   * Supporting method for getting proper format of test method name
   *
   * @param ec ExtensionContext
   * @return package.Test > displayName
   */
  public static String getTestFullName(ExtensionContext ec) {
    String description = ec.getTestMethod().get().toString();
    String[] parts = description.split("\\s+");
    // void testpackage.test
    String fullPaths = parts[1];
    int index = fullPaths.indexOf("TS_");
    if (index < 0 && parts.length > 2) {
      // public void testpackage.test
      fullPaths = parts[2];
      index = fullPaths.indexOf("TS_");
    }
    String testName = description;
    if (index > 0) {
      index = fullPaths.indexOf(".", index);
      testName = fullPaths.substring(0, index);
    }
    String displayName = ec.getDisplayName();
    if (displayName.equals("testTemplate") == true) {
      String grandParent = ec.getParent().get().getParent().get().getDisplayName();
      int i = grandParent.indexOf(":");
      grandParent = grandParent.substring(0, i);
      displayName = grandParent + ">>" + ec.getParent().get().getDisplayName();
    }
    return testName + " > " + displayName;
  }

  /**
   * Supporting method for getting proper format of an error message
   *
   * @param description error description with white chars
   * @return reformed error message suitable for logging
   */
  private String getExceptionMessage(String description) {
    String message = description.replaceAll("\\v", "===>");
    message = message.replaceAll("\t", " ");
    message = message.replaceAll("===>===>", " ===>");
    return message;
  }

}