package pstsp.support.test_utils;

/**
 * "setting" class of all test
 * activates drivers, logger, screen saver etc.
 *
 * Warning - do not inherite it!
 *
 * MUST be activated in ANY testing file by:
 *
 * @code{@ExtendWith(TestSetting.class)
         public class NameOfClassTest }
 *
 * It guarantees just one-time initialization of a web driver
 * and all supporting oracle content
 * before the very first test
 * and its termination after very last test
 *
 * The same for logging and screenshot saving
 *
 * @author Pavel Herout
 * @version 2019-24-11
 */

import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.BeforeTestExecutionCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import pstsp.support.basic.Configurations;
import pstsp.support.tools.Drivers;
import pstsp.support.tools.ProjectLogger;

import static org.junit.jupiter.api.extension.ExtensionContext.Namespace.GLOBAL;

public class TestSetting implements BeforeAllCallback, BeforeTestExecutionCallback,
        ExtensionContext.Store.CloseableResource {

  /** temporally variable assuring, that initialization will run just once only */
  private static boolean started = false;

  /** number of all tests - for continuous logging report */
  private static int testNumber = 0;


  /**
   * One-time initialization of web driver
   * Possible several others initializations, eg. Logger etc.
   *
   * the string "Hello" is a unique string and it is never used anywhere
   * in this whole project
   *
   * @param context parameter supplied by Selenium
   */
  @Override
  public void beforeAll(ExtensionContext context) {
    if (started == false) {
      started = true;
      context.getRoot().getStore(GLOBAL).put("Hello", this);
      Configurations.getInstance().setConfiguration();
      TestEvaluation.clearCounters();
      ProjectLogger.logStartInfo();
    }
  }

  /**
   * Termination of web driver after very last test
   * Possibly several others final activities, eg. Logger etc.
   */
  @Override
  public void close() {
    if (Drivers.CLOSE_BROWSER.equals("true")) {
      Drivers.quitWebDriver();
    }
    ProjectLogger.logEndInfo();
    started = false;
  }

  /**
   * Callback activity before each test execution
   * it is used for logging and for possible screenshot saving
   *
   * @param ec parameter supplied by Selenium
   */
  @Override
  public void beforeTestExecution(ExtensionContext ec) {
    testNumber++;
    String testNumberStr = String.format("%04d. ", testNumber);
    String message = testNumberStr +
               TestEvaluation.getTestFullName(ec);
    ProjectLogger.logger.info(message);
  }

}