package pstsp.support.tools;

/**
 * Logs result of tests according to logger config file, which name is defined
 * in the configuration file (default: test-results-log.txt)
 *
 * see {link TestEvaluation} class for logged information
 * Passed test: INFO level
 * Failed test: ERROR level
 *
 * During the configuration, this activity is loggen into another file,
 * which name is defined in the configuration file too
 * (default: test-config-log.txt)
 *
 * Levels: DEBUG and possibly (needs a change in the config file) TRACE
 *
 * @author Pavel Herout
 * @version 2022-01-20
 */

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import pstsp.support.basic.Configurations;
import pstsp.support.basic.Url;
import pstsp.support.test_utils.TestEvaluation;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;

public class ProjectLogger {

  /** the value from CONFIGURATIONS_FILE
   * name of logger config file, this file must be stored at the same directory
   * as the configuration file
   */
  public static String LOG_FILE_CONFIG;

  /** the instance of only one logger */
  public static Logger logger;

  /** date & time of the beginning of all tests */
  public static String TEST_DATE_TIME = "not set yet";

  /** date & time of beginning of tests - it serves for log elapsed time calculating */
  private static LocalDateTime ldtStart;

  /**
   * Setting of logger conditions
   *
   * @param configDir configuration directory
   */
  public static void setLogger(String configDir) {
    String file = "file:///" + configDir + LOG_FILE_CONFIG;
    System.setProperty("log4j.configurationFile", file);
    logger = LogManager.getLogger("PSTSP");
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd_HH-mm");
    TEST_DATE_TIME = LocalDateTime.now().format(formatter);
  }

  /**
   * logs the basic starting information ("headline") about actually run tests
   */
  public static void logStartInfo() {
    ldtStart = LocalDateTime.now();

    logger.info("=== Basic info =========================================");
    logger.info("=== Browser:              " + Drivers.getBrowserType());
    logger.info("=== Date_time:            " + TEST_DATE_TIME);
    logger.info("=== Base URL:             " + Url.BASE_URL);
    logger.info("=== Build number:         " + Configurations.BUILD_NUMBER);
    logger.info("end Basic info =========================================");
  }

  /**
   * logs the basic ending information about actually run tests
   */
  public static void logEndInfo() {
    int testPassed = TestEvaluation.getTestPassed();
    int testFailed = TestEvaluation.getTestFailed();
    int testDisabled = TestEvaluation.getTestDisabled();
    int testAborted = TestEvaluation.getTestAborted();
    int numberOfAsserts = TestEvaluation.getNumberOfAsserts();
    int testTotal = testPassed + testFailed + testDisabled + testAborted;

    logger.info("=== Total tests:    " + String.format("%4d", testTotal));
    logger.info("=== Tests failed:   " + String.format("%4d", testFailed));
    if (testDisabled != 0) {
      logger.info("=== Tests disabled: " + String.format("%4d", testDisabled));
    }
    if (testAborted != 0) {
      logger.info("=== Tests aborted:   " + String.format("%4d", testAborted));
    }
    logger.info("=== Total asserts:  " + String.format("%4d", numberOfAsserts));
    LocalDateTime ldtEnd = LocalDateTime.now();
    long diff = ChronoUnit.SECONDS.between(ldtStart, ldtEnd);
    logger.info("=== Elapsed time:   " + String.format("%4d", diff) + " [s]");
  }

  /**
   * Return date + time of the beginning of all tests
   * @return date + time
   */
  public static String getTestDateTime() {
    return TEST_DATE_TIME;
  }

}
